print("import: 'cookiecutter'")
import cookiecutter

print("import: 'cookiecutter.main'")
import cookiecutter.main

print("import: 'cookiecutter.environment'")
import cookiecutter.environment

print("import: 'cookiecutter.exceptions'")
import cookiecutter.exceptions

